#ifndef POINT_HPP
#define POINT_HPP

#include <cmath>
#include <iostream>
#include "cuda.hpp"

struct Point {
   double x;
   double y;
   double z;
   HOST_DEV Point() : x(0), y(0), z(0) {
   }
   HOST_DEV Point(double x, double y, double z) : x(x), y(y), z(z) {
   }
};

struct Vector {
   double x;
   double y;
   double z;
   HOST_DEV Vector() : x(0), y(0), z(0) {
   }
   HOST_DEV Vector(double x, double y, double z) : x(x), y(y), z(z) {
   }
   HOST_DEV double length() const {
      return sqrt(x * x + y * y + z * z);
   }
   HOST_DEV void normalize() {
      double len = length();
      x /= len; y /= len; z /= len;
   }
};

HOST_DEV static inline Point operator+(const Point& p, const Vector& v) {
   return Point(p.x + v.x, p.y + v.y, p.z + v.z);
}
HOST_DEV static inline Point operator-(const Point& p, const Vector& v) {
   return Point(p.x - v.x, p.y - v.y, p.z - v.z);
}
HOST_DEV static inline Vector operator+(const Vector& a, const Vector& b) {
   return Vector(a.x + b.x, a.y + b.y, a.z + b.z);
}
HOST_DEV static inline Vector operator-(const Point& a, const Point& b) {
   return Vector(a.x - b.x, a.y - b.y, a.z - b.z);
}
HOST_DEV static inline Vector operator*(double alpha, const Vector& a) {
   return Vector(alpha * a.x, alpha * a.y, alpha * a.z);
}
HOST_DEV static inline Vector operator*(const Vector& a, double alpha) {
   return Vector(alpha * a.x, alpha * a.y, alpha * a.z);
}
HOST_DEV static inline Vector operator/(const Vector& a, double alpha) {
   return Vector(a.x / alpha, a.y / alpha, a.z / alpha);
}
HOST_DEV static inline double operator*(const Vector& a, const Vector& b) {
   return a.x * b.x + a.y * b.y + a.z * b.z; 
}
HOST_DEV static inline Vector normalize(const Vector& v) {
   Vector w = v; w.normalize(); return w;
}

std::ostream& operator<<(std::ostream& out, const Point& p) {
   return std::cout << "(" << p.x << ", " << p.y << ", " << p.z << ")";
}

std::ostream& operator<<(std::ostream& out, const Vector& v) {
   return std::cout << "(" << v.x << ", " << v.y << ", " << v.z << ")";
}

#endif
