#ifndef RGBCOLOR_HPP
#define RGBCOLOR_HPP

#include <cfloat>
#include <cmath>
#include <iostream>
#include "cuda.hpp"

struct RGBColor {
   HOST_DEV RGBColor() : red(0), green(0), blue(0) {
   }
   HOST_DEV RGBColor(float red, float green, float blue) :
      red(red), green(green), blue(blue) {
   }
   RGBColor get_rgb() const {
      return *this;
   }
   RGBColor interpolate(const RGBColor& other, float t) {
      return RGBColor(
	 (red + (other.red - red) * t),
	 (green + (other.green - green) * t),
	 (blue + (other.blue - blue) * t));
   }
   bool operator==(const RGBColor& other) {
      return std::fabs(red - other.red) < DBL_EPSILON &&
             std::fabs(green - other.green) < DBL_EPSILON &&
	     std::fabs(blue - other.blue) < DBL_EPSILON;
   }
   bool operator!=(const RGBColor& other) {
      return !(*this == other);
   }
   float red;
   float green;
   float blue;
};

HOST_DEV static inline
RGBColor operator*(const RGBColor& color, double factor) {
   factor = std::fmod(std::fabs(factor), 1.0);
   return RGBColor(color.red * factor, color.green * factor,
      color.blue * factor);
}

inline std::ostream& operator<<(std::ostream& out, const RGBColor& color) {
   return out << "(" << color.red << ", " << color.green <<
      ", " << color.blue << ")";
}

#endif
