#ifndef SPHERE_H
#define SPHERE_H

#include <cmath>
#include <iostream>
#include "cuda.hpp"
#include "point.hpp"
#include "rgbcolor.hpp"
#include "shade.hpp"

#define EPSILON 1E-10

struct Sphere {
   HOST_DEV Sphere() : center(0, 0, 0), radius(1), color(0, 0, 0) {
   }
   HOST_DEV Sphere(const Point& center, double radius, RGBColor color) :
	 center(center), radius(radius), color(color) {
   }
   HOST_DEV bool hit(const Ray& ray, double& tmin, Shade& shade) {
      double a = ray.direction * ray.direction;
      Vector delta = ray.origin - center;
      double b = 2.0 * delta * ray.direction;
      double c = delta * delta - radius * radius;
      double d = b * b - 4.0 * a * c;
      if (d < 0.0) return false;
      double e = sqrt(d);
      double denominator = 2 * a;
      double t = (-b - e) / denominator;
      if (t < EPSILON) {
	 t = (-b + e) / denominator;
      }
      if (t < EPSILON) return false;
      /* hit */
      tmin = t;
      shade.normal = (delta + t * ray.direction) / radius;
      shade.hit = ray.origin + t * ray.direction;
      shade.color = color;
      return true;
   }
   Point center;
   double radius;
   RGBColor color;
};

std::ostream& operator<<(std::ostream& out, const Sphere& sphere) {
   return out << "sphere at " << sphere.center << " with radius "
      << sphere.radius << " and color " << sphere.color;
}

bool intersect(const Sphere& a, const Sphere& b) {
   double distance = (a.center - b.center).length();
   return a.radius + b.radius >= distance;
}

#endif
